/*
 * nghttp2 - HTTP/2.0 C Library
 *
 * Copyright (c) 2013 Tatsuhiro Tsujikawa
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include "nghttp2_hd_huffman.h"

const nghttp2_huff_sym req_huff_sym_table[] = {
  { 27, 0x7ffffbau },
  { 27, 0x7ffffbbu },
  { 27, 0x7ffffbcu },
  { 27, 0x7ffffbdu },
  { 27, 0x7ffffbeu },
  { 27, 0x7ffffbfu },
  { 27, 0x7ffffc0u },
  { 27, 0x7ffffc1u },
  { 27, 0x7ffffc2u },
  { 27, 0x7ffffc3u },
  { 27, 0x7ffffc4u },
  { 27, 0x7ffffc5u },
  { 27, 0x7ffffc6u },
  { 27, 0x7ffffc7u },
  { 27, 0x7ffffc8u },
  { 27, 0x7ffffc9u },
  { 27, 0x7ffffcau },
  { 27, 0x7ffffcbu },
  { 27, 0x7ffffccu },
  { 27, 0x7ffffcdu },
  { 27, 0x7ffffceu },
  { 27, 0x7ffffcfu },
  { 27, 0x7ffffd0u },
  { 27, 0x7ffffd1u },
  { 27, 0x7ffffd2u },
  { 27, 0x7ffffd3u },
  { 27, 0x7ffffd4u },
  { 27, 0x7ffffd5u },
  { 27, 0x7ffffd6u },
  { 27, 0x7ffffd7u },
  { 27, 0x7ffffd8u },
  { 27, 0x7ffffd9u },
  { 8, 0xe8u },
  { 12, 0xffcu },
  { 14, 0x3ffau },
  { 15, 0x7ffcu },
  { 15, 0x7ffdu },
  { 6, 0x24u },
  { 7, 0x6eu },
  { 15, 0x7ffeu },
  { 11, 0x7fau },
  { 11, 0x7fbu },
  { 10, 0x3fau },
  { 11, 0x7fcu },
  { 8, 0xe9u },
  { 6, 0x25u },
  { 5, 0x4u },
  { 4, 0x0u },
  { 5, 0x5u },
  { 5, 0x6u },
  { 5, 0x7u },
  { 6, 0x26u },
  { 6, 0x27u },
  { 6, 0x28u },
  { 6, 0x29u },
  { 6, 0x2au },
  { 6, 0x2bu },
  { 6, 0x2cu },
  { 9, 0x1ecu },
  { 8, 0xeau },
  { 18, 0x3fffeu },
  { 6, 0x2du },
  { 17, 0x1fffcu },
  { 9, 0x1edu },
  { 14, 0x3ffbu },
  { 7, 0x6fu },
  { 8, 0xebu },
  { 8, 0xecu },
  { 8, 0xedu },
  { 8, 0xeeu },
  { 7, 0x70u },
  { 9, 0x1eeu },
  { 9, 0x1efu },
  { 9, 0x1f0u },
  { 9, 0x1f1u },
  { 10, 0x3fbu },
  { 9, 0x1f2u },
  { 8, 0xefu },
  { 9, 0x1f3u },
  { 9, 0x1f4u },
  { 9, 0x1f5u },
  { 9, 0x1f6u },
  { 9, 0x1f7u },
  { 8, 0xf0u },
  { 8, 0xf1u },
  { 9, 0x1f8u },
  { 9, 0x1f9u },
  { 9, 0x1fau },
  { 9, 0x1fbu },
  { 9, 0x1fcu },
  { 10, 0x3fcu },
  { 14, 0x3ffcu },
  { 27, 0x7ffffdau },
  { 13, 0x1ffcu },
  { 14, 0x3ffdu },
  { 6, 0x2eu },
  { 19, 0x7fffeu },
  { 5, 0x8u },
  { 6, 0x2fu },
  { 5, 0x9u },
  { 6, 0x30u },
  { 4, 0x1u },
  { 6, 0x31u },
  { 6, 0x32u },
  { 6, 0x33u },
  { 5, 0xau },
  { 7, 0x71u },
  { 7, 0x72u },
  { 5, 0xbu },
  { 6, 0x34u },
  { 5, 0xcu },
  { 5, 0xdu },
  { 5, 0xeu },
  { 8, 0xf2u },
  { 5, 0xfu },
  { 5, 0x10u },
  { 5, 0x11u },
  { 6, 0x35u },
  { 7, 0x73u },
  { 6, 0x36u },
  { 8, 0xf3u },
  { 8, 0xf4u },
  { 8, 0xf5u },
  { 17, 0x1fffdu },
  { 11, 0x7fdu },
  { 17, 0x1fffeu },
  { 12, 0xffdu },
  { 27, 0x7ffffdbu },
  { 27, 0x7ffffdcu },
  { 27, 0x7ffffddu },
  { 27, 0x7ffffdeu },
  { 27, 0x7ffffdfu },
  { 27, 0x7ffffe0u },
  { 27, 0x7ffffe1u },
  { 27, 0x7ffffe2u },
  { 27, 0x7ffffe3u },
  { 27, 0x7ffffe4u },
  { 27, 0x7ffffe5u },
  { 27, 0x7ffffe6u },
  { 27, 0x7ffffe7u },
  { 27, 0x7ffffe8u },
  { 27, 0x7ffffe9u },
  { 27, 0x7ffffeau },
  { 27, 0x7ffffebu },
  { 27, 0x7ffffecu },
  { 27, 0x7ffffedu },
  { 27, 0x7ffffeeu },
  { 27, 0x7ffffefu },
  { 27, 0x7fffff0u },
  { 27, 0x7fffff1u },
  { 27, 0x7fffff2u },
  { 27, 0x7fffff3u },
  { 27, 0x7fffff4u },
  { 27, 0x7fffff5u },
  { 27, 0x7fffff6u },
  { 27, 0x7fffff7u },
  { 27, 0x7fffff8u },
  { 27, 0x7fffff9u },
  { 27, 0x7fffffau },
  { 27, 0x7fffffbu },
  { 27, 0x7fffffcu },
  { 27, 0x7fffffdu },
  { 27, 0x7fffffeu },
  { 27, 0x7ffffffu },
  { 26, 0x3ffff80u },
  { 26, 0x3ffff81u },
  { 26, 0x3ffff82u },
  { 26, 0x3ffff83u },
  { 26, 0x3ffff84u },
  { 26, 0x3ffff85u },
  { 26, 0x3ffff86u },
  { 26, 0x3ffff87u },
  { 26, 0x3ffff88u },
  { 26, 0x3ffff89u },
  { 26, 0x3ffff8au },
  { 26, 0x3ffff8bu },
  { 26, 0x3ffff8cu },
  { 26, 0x3ffff8du },
  { 26, 0x3ffff8eu },
  { 26, 0x3ffff8fu },
  { 26, 0x3ffff90u },
  { 26, 0x3ffff91u },
  { 26, 0x3ffff92u },
  { 26, 0x3ffff93u },
  { 26, 0x3ffff94u },
  { 26, 0x3ffff95u },
  { 26, 0x3ffff96u },
  { 26, 0x3ffff97u },
  { 26, 0x3ffff98u },
  { 26, 0x3ffff99u },
  { 26, 0x3ffff9au },
  { 26, 0x3ffff9bu },
  { 26, 0x3ffff9cu },
  { 26, 0x3ffff9du },
  { 26, 0x3ffff9eu },
  { 26, 0x3ffff9fu },
  { 26, 0x3ffffa0u },
  { 26, 0x3ffffa1u },
  { 26, 0x3ffffa2u },
  { 26, 0x3ffffa3u },
  { 26, 0x3ffffa4u },
  { 26, 0x3ffffa5u },
  { 26, 0x3ffffa6u },
  { 26, 0x3ffffa7u },
  { 26, 0x3ffffa8u },
  { 26, 0x3ffffa9u },
  { 26, 0x3ffffaau },
  { 26, 0x3ffffabu },
  { 26, 0x3ffffacu },
  { 26, 0x3ffffadu },
  { 26, 0x3ffffaeu },
  { 26, 0x3ffffafu },
  { 26, 0x3ffffb0u },
  { 26, 0x3ffffb1u },
  { 26, 0x3ffffb2u },
  { 26, 0x3ffffb3u },
  { 26, 0x3ffffb4u },
  { 26, 0x3ffffb5u },
  { 26, 0x3ffffb6u },
  { 26, 0x3ffffb7u },
  { 26, 0x3ffffb8u },
  { 26, 0x3ffffb9u },
  { 26, 0x3ffffbau },
  { 26, 0x3ffffbbu },
  { 26, 0x3ffffbcu },
  { 26, 0x3ffffbdu },
  { 26, 0x3ffffbeu },
  { 26, 0x3ffffbfu },
  { 26, 0x3ffffc0u },
  { 26, 0x3ffffc1u },
  { 26, 0x3ffffc2u },
  { 26, 0x3ffffc3u },
  { 26, 0x3ffffc4u },
  { 26, 0x3ffffc5u },
  { 26, 0x3ffffc6u },
  { 26, 0x3ffffc7u },
  { 26, 0x3ffffc8u },
  { 26, 0x3ffffc9u },
  { 26, 0x3ffffcau },
  { 26, 0x3ffffcbu },
  { 26, 0x3ffffccu },
  { 26, 0x3ffffcdu },
  { 26, 0x3ffffceu },
  { 26, 0x3ffffcfu },
  { 26, 0x3ffffd0u },
  { 26, 0x3ffffd1u },
  { 26, 0x3ffffd2u },
  { 26, 0x3ffffd3u },
  { 26, 0x3ffffd4u },
  { 26, 0x3ffffd5u },
  { 26, 0x3ffffd6u },
  { 26, 0x3ffffd7u },
  { 26, 0x3ffffd8u },
  { 26, 0x3ffffd9u },
  { 26, 0x3ffffdau },
  { 26, 0x3ffffdbu },
  { 26, 0x3ffffdcu }
};

const int16_t req_huff_decode_table[][256] = {
/* 0 */
{
 47, 47, 47, 47, 47, 47, 47, 47,
 47, 47, 47, 47, 47, 47, 47, 47,
 101, 101, 101, 101, 101, 101, 101, 101,
 101, 101, 101, 101, 101, 101, 101, 101,
 46, 46, 46, 46, 46, 46, 46, 46,
 48, 48, 48, 48, 48, 48, 48, 48,
 49, 49, 49, 49, 49, 49, 49, 49,
 50, 50, 50, 50, 50, 50, 50, 50,
 97, 97, 97, 97, 97, 97, 97, 97,
 99, 99, 99, 99, 99, 99, 99, 99,
 105, 105, 105, 105, 105, 105, 105, 105,
 108, 108, 108, 108, 108, 108, 108, 108,
 110, 110, 110, 110, 110, 110, 110, 110,
 111, 111, 111, 111, 111, 111, 111, 111,
 112, 112, 112, 112, 112, 112, 112, 112,
 114, 114, 114, 114, 114, 114, 114, 114,
 115, 115, 115, 115, 115, 115, 115, 115,
 116, 116, 116, 116, 116, 116, 116, 116,
 37, 37, 37, 37, 45, 45, 45, 45,
 51, 51, 51, 51, 52, 52, 52, 52,
 53, 53, 53, 53, 54, 54, 54, 54,
 55, 55, 55, 55, 56, 56, 56, 56,
 57, 57, 57, 57, 61, 61, 61, 61,
 95, 95, 95, 95, 98, 98, 98, 98,
 100, 100, 100, 100, 102, 102, 102, 102,
 103, 103, 103, 103, 104, 104, 104, 104,
 109, 109, 109, 109, 117, 117, 117, 117,
 119, 119, 119, 119, 38, 38, 65, 65,
 70, 70, 106, 106, 107, 107, 118, 118,
 32, 44, 59, 66, 67, 68, 69, 77,
 83, 84, 113, 120, 121, 122, -9, -11,
 -12, -13, -14, -15, -16, -17, -8, -1
},
/* 1 */
{
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 124, 124, 124, 124, 124, 124, 124, 124,
 124, 124, 124, 124, 124, 124, 124, 124,
 124, 124, 124, 124, 124, 124, 124, 124,
 124, 124, 124, 124, 124, 124, 124, 124,
 33, 33, 33, 33, 33, 33, 33, 33,
 33, 33, 33, 33, 33, 33, 33, 33,
 126, 126, 126, 126, 126, 126, 126, 126,
 126, 126, 126, 126, 126, 126, 126, 126,
 93, 93, 93, 93, 93, 93, 93, 93,
 34, 34, 34, 34, 64, 64, 64, 64,
 91, 91, 91, 91, 94, 94, 94, 94,
 35, 35, 36, 36, 39, 39, -10, -2
},
/* 2 */
{
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 125, 125, 125, 125, 125, 125, 125, 125,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 60, 60, 60, 60, 60, 60, 60, 60,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 -22, -23, -24, -25, -26, -27, -28, -29,
 -30, -31, -32, -33, -34, -35, -36, -37,
 -38, -39, -40, -41, -42, -43, -44, -3,
 -4, -5, -6, -7, -18, -19, -20, -21
},
/* 3 */
{
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 256, 256, 256, 256, 256, 256, 256, 256,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5
},
/* 4 */
{
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13
},
/* 5 */
{
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21
},
/* 6 */
{
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29
},
/* 7 */
{
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 92, 92, 92, 92, 92, 92, 92, 92,
 92, 92, 92, 92, 92, 92, 92, 92,
 92, 92, 92, 92, 92, 92, 92, 92,
 92, 92, 92, 92, 92, 92, 92, 92,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131
},
/* 8 */
{
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75
},
/* 9 */
{
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 58, 58, 58, 58, 58, 58, 58, 58,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63
},
/* 10 */
{
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 62, 62, 62, 62, 62, 62, 62, 62,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123, 123, 123, 123, 123, 123
},
/* 11 */
{
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 71, 71, 71, 71, 71, 71, 71, 71,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72
},
/* 12 */
{
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 73, 73, 73, 73, 73, 73, 73, 73,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74,
 74, 74, 74, 74, 74, 74, 74, 74
},
/* 13 */
{
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78,
 78, 78, 78, 78, 78, 78, 78, 78
},
/* 14 */
{
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 79, 79, 79, 79, 79, 79, 79, 79,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80
},
/* 15 */
{
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82
},
/* 16 */
{
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86
},
/* 17 */
{
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 87, 87, 87, 87, 87, 87, 87, 87,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88
},
/* 18 */
{
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139
},
/* 19 */
{
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147
},
/* 20 */
{
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155
},
/* 21 */
{
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 163, 163, 163, 163, 163, 163, 163, 163,
 163, 163, 163, 163, 163, 163, 163, 163,
 163, 163, 163, 163, 163, 163, 163, 163,
 163, 163, 163, 163, 163, 163, 163, 163
},
/* 22 */
{
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 164, 164, 164, 164, 164, 164, 164, 164,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 165, 165, 165, 165, 165, 165, 165, 165,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 166, 166, 166, 166, 166, 166, 166, 166,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167,
 167, 167, 167, 167, 167, 167, 167, 167
},
/* 23 */
{
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 168, 168, 168, 168, 168, 168, 168, 168,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 169, 169, 169, 169, 169, 169, 169, 169,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 170, 170, 170, 170, 170, 170, 170, 170,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171,
 171, 171, 171, 171, 171, 171, 171, 171
},
/* 24 */
{
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 172, 172, 172, 172, 172, 172, 172, 172,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 173, 173, 173, 173, 173, 173, 173, 173,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 174, 174, 174, 174, 174, 174, 174, 174,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175,
 175, 175, 175, 175, 175, 175, 175, 175
},
/* 25 */
{
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 176, 176, 176, 176, 176, 176, 176, 176,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 177, 177, 177, 177, 177, 177, 177, 177,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 178, 178, 178, 178, 178, 178, 178, 178,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179,
 179, 179, 179, 179, 179, 179, 179, 179
},
/* 26 */
{
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 180, 180, 180, 180, 180, 180, 180, 180,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 181, 181, 181, 181, 181, 181, 181, 181,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 182, 182, 182, 182, 182, 182, 182, 182,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183,
 183, 183, 183, 183, 183, 183, 183, 183
},
/* 27 */
{
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 184, 184, 184, 184, 184, 184, 184, 184,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187,
 187, 187, 187, 187, 187, 187, 187, 187
},
/* 28 */
{
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 188, 188, 188, 188, 188, 188, 188, 188,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 189, 189, 189, 189, 189, 189, 189, 189,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 190, 190, 190, 190, 190, 190, 190, 190,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191,
 191, 191, 191, 191, 191, 191, 191, 191
},
/* 29 */
{
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 192, 192, 192, 192, 192, 192, 192, 192,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 193, 193, 193, 193, 193, 193, 193, 193,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 194, 194, 194, 194, 194, 194, 194, 194,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195,
 195, 195, 195, 195, 195, 195, 195, 195
},
/* 30 */
{
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 196, 196, 196, 196, 196, 196, 196, 196,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 197, 197, 197, 197, 197, 197, 197, 197,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 198, 198, 198, 198, 198, 198, 198, 198,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199,
 199, 199, 199, 199, 199, 199, 199, 199
},
/* 31 */
{
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 200, 200, 200, 200, 200, 200, 200, 200,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 201, 201, 201, 201, 201, 201, 201, 201,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 202, 202, 202, 202, 202, 202, 202, 202,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203,
 203, 203, 203, 203, 203, 203, 203, 203
},
/* 32 */
{
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 204, 204, 204, 204, 204, 204, 204, 204,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 205, 205, 205, 205, 205, 205, 205, 205,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 206, 206, 206, 206, 206, 206, 206, 206,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207,
 207, 207, 207, 207, 207, 207, 207, 207
},
/* 33 */
{
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 208, 208, 208, 208, 208, 208, 208, 208,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 209, 209, 209, 209, 209, 209, 209, 209,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 210, 210, 210, 210, 210, 210, 210, 210,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211,
 211, 211, 211, 211, 211, 211, 211, 211
},
/* 34 */
{
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 212, 212, 212, 212, 212, 212, 212, 212,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 213, 213, 213, 213, 213, 213, 213, 213,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 214, 214, 214, 214, 214, 214, 214, 214,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215,
 215, 215, 215, 215, 215, 215, 215, 215
},
/* 35 */
{
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 216, 216, 216, 216, 216, 216, 216, 216,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 217, 217, 217, 217, 217, 217, 217, 217,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 218, 218, 218, 218, 218, 218, 218, 218,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219,
 219, 219, 219, 219, 219, 219, 219, 219
},
/* 36 */
{
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 220, 220, 220, 220, 220, 220, 220, 220,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 221, 221, 221, 221, 221, 221, 221, 221,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 222, 222, 222, 222, 222, 222, 222, 222,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223,
 223, 223, 223, 223, 223, 223, 223, 223
},
/* 37 */
{
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 224, 224, 224, 224, 224, 224, 224, 224,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 225, 225, 225, 225, 225, 225, 225, 225,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 226, 226, 226, 226, 226, 226, 226, 226,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227,
 227, 227, 227, 227, 227, 227, 227, 227
},
/* 38 */
{
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 228, 228, 228, 228, 228, 228, 228, 228,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 229, 229, 229, 229, 229, 229, 229, 229,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 230, 230, 230, 230, 230, 230, 230, 230,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231,
 231, 231, 231, 231, 231, 231, 231, 231
},
/* 39 */
{
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 232, 232, 232, 232, 232, 232, 232, 232,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 233, 233, 233, 233, 233, 233, 233, 233,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 234, 234, 234, 234, 234, 234, 234, 234,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235,
 235, 235, 235, 235, 235, 235, 235, 235
},
/* 40 */
{
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 236, 236, 236, 236, 236, 236, 236, 236,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 237, 237, 237, 237, 237, 237, 237, 237,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 238, 238, 238, 238, 238, 238, 238, 238,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239,
 239, 239, 239, 239, 239, 239, 239, 239
},
/* 41 */
{
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 240, 240, 240, 240, 240, 240, 240, 240,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 241, 241, 241, 241, 241, 241, 241, 241,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 242, 242, 242, 242, 242, 242, 242, 242,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243,
 243, 243, 243, 243, 243, 243, 243, 243
},
/* 42 */
{
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 244, 244, 244, 244, 244, 244, 244, 244,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 245, 245, 245, 245, 245, 245, 245, 245,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 246, 246, 246, 246, 246, 246, 246, 246,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247,
 247, 247, 247, 247, 247, 247, 247, 247
},
/* 43 */
{
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 248, 248, 248, 248, 248, 248, 248, 248,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 249, 249, 249, 249, 249, 249, 249, 249,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 250, 250, 250, 250, 250, 250, 250, 250,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251,
 251, 251, 251, 251, 251, 251, 251, 251
},
/* 44 */
{
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 252, 252, 252, 252, 252, 252, 252, 252,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 253, 253, 253, 253, 253, 253, 253, 253,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 254, 254, 254, 254, 254, 254, 254, 254,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255,
 255, 255, 255, 255, 255, 255, 255, 255
}
};

const nghttp2_huff_sym res_huff_sym_table[] = {
  { 25, 0x1ffffbcu },
  { 25, 0x1ffffbdu },
  { 25, 0x1ffffbeu },
  { 25, 0x1ffffbfu },
  { 25, 0x1ffffc0u },
  { 25, 0x1ffffc1u },
  { 25, 0x1ffffc2u },
  { 25, 0x1ffffc3u },
  { 25, 0x1ffffc4u },
  { 25, 0x1ffffc5u },
  { 25, 0x1ffffc6u },
  { 25, 0x1ffffc7u },
  { 25, 0x1ffffc8u },
  { 25, 0x1ffffc9u },
  { 25, 0x1ffffcau },
  { 25, 0x1ffffcbu },
  { 25, 0x1ffffccu },
  { 25, 0x1ffffcdu },
  { 25, 0x1ffffceu },
  { 25, 0x1ffffcfu },
  { 25, 0x1ffffd0u },
  { 25, 0x1ffffd1u },
  { 25, 0x1ffffd2u },
  { 25, 0x1ffffd3u },
  { 25, 0x1ffffd4u },
  { 25, 0x1ffffd5u },
  { 25, 0x1ffffd6u },
  { 25, 0x1ffffd7u },
  { 25, 0x1ffffd8u },
  { 25, 0x1ffffd9u },
  { 25, 0x1ffffdau },
  { 25, 0x1ffffdbu },
  { 4, 0x0u },
  { 12, 0xffau },
  { 7, 0x6au },
  { 13, 0x1ffau },
  { 14, 0x3ffcu },
  { 9, 0x1ecu },
  { 10, 0x3f8u },
  { 13, 0x1ffbu },
  { 9, 0x1edu },
  { 9, 0x1eeu },
  { 12, 0xffbu },
  { 11, 0x7fau },
  { 6, 0x22u },
  { 6, 0x23u },
  { 6, 0x24u },
  { 7, 0x6bu },
  { 4, 0x1u },
  { 4, 0x2u },
  { 4, 0x3u },
  { 5, 0x8u },
  { 5, 0x9u },
  { 5, 0xau },
  { 6, 0x25u },
  { 6, 0x26u },
  { 5, 0xbu },
  { 5, 0xcu },
  { 5, 0xdu },
  { 9, 0x1efu },
  { 16, 0xfffau },
  { 7, 0x6cu },
  { 13, 0x1ffcu },
  { 12, 0xffcu },
  { 16, 0xfffbu },
  { 7, 0x6du },
  { 8, 0xeau },
  { 8, 0xebu },
  { 8, 0xecu },
  { 8, 0xedu },
  { 8, 0xeeu },
  { 6, 0x27u },
  { 9, 0x1f0u },
  { 8, 0xefu },
  { 8, 0xf0u },
  { 10, 0x3f9u },
  { 9, 0x1f1u },
  { 6, 0x28u },
  { 8, 0xf1u },
  { 8, 0xf2u },
  { 9, 0x1f2u },
  { 10, 0x3fau },
  { 9, 0x1f3u },
  { 6, 0x29u },
  { 5, 0xeu },
  { 9, 0x1f4u },
  { 9, 0x1f5u },
  { 8, 0xf3u },
  { 10, 0x3fbu },
  { 9, 0x1f6u },
  { 10, 0x3fcu },
  { 11, 0x7fbu },
  { 13, 0x1ffdu },
  { 11, 0x7fcu },
  { 15, 0x7ffcu },
  { 9, 0x1f7u },
  { 17, 0x1fffeu },
  { 5, 0xfu },
  { 7, 0x6eu },
  { 6, 0x2au },
  { 6, 0x2bu },
  { 5, 0x10u },
  { 7, 0x6fu },
  { 7, 0x70u },
  { 7, 0x71u },
  { 6, 0x2cu },
  { 9, 0x1f8u },
  { 9, 0x1f9u },
  { 7, 0x72u },
  { 6, 0x2du },
  { 6, 0x2eu },
  { 6, 0x2fu },
  { 6, 0x30u },
  { 9, 0x1fau },
  { 6, 0x31u },
  { 6, 0x32u },
  { 6, 0x33u },
  { 6, 0x34u },
  { 7, 0x73u },
  { 8, 0xf4u },
  { 7, 0x74u },
  { 8, 0xf5u },
  { 9, 0x1fbu },
  { 16, 0xfffcu },
  { 14, 0x3ffdu },
  { 16, 0xfffdu },
  { 16, 0xfffeu },
  { 25, 0x1ffffdcu },
  { 25, 0x1ffffddu },
  { 25, 0x1ffffdeu },
  { 25, 0x1ffffdfu },
  { 25, 0x1ffffe0u },
  { 25, 0x1ffffe1u },
  { 25, 0x1ffffe2u },
  { 25, 0x1ffffe3u },
  { 25, 0x1ffffe4u },
  { 25, 0x1ffffe5u },
  { 25, 0x1ffffe6u },
  { 25, 0x1ffffe7u },
  { 25, 0x1ffffe8u },
  { 25, 0x1ffffe9u },
  { 25, 0x1ffffeau },
  { 25, 0x1ffffebu },
  { 25, 0x1ffffecu },
  { 25, 0x1ffffedu },
  { 25, 0x1ffffeeu },
  { 25, 0x1ffffefu },
  { 25, 0x1fffff0u },
  { 25, 0x1fffff1u },
  { 25, 0x1fffff2u },
  { 25, 0x1fffff3u },
  { 25, 0x1fffff4u },
  { 25, 0x1fffff5u },
  { 25, 0x1fffff6u },
  { 25, 0x1fffff7u },
  { 25, 0x1fffff8u },
  { 25, 0x1fffff9u },
  { 25, 0x1fffffau },
  { 25, 0x1fffffbu },
  { 25, 0x1fffffcu },
  { 25, 0x1fffffdu },
  { 25, 0x1fffffeu },
  { 25, 0x1ffffffu },
  { 24, 0xffff80u },
  { 24, 0xffff81u },
  { 24, 0xffff82u },
  { 24, 0xffff83u },
  { 24, 0xffff84u },
  { 24, 0xffff85u },
  { 24, 0xffff86u },
  { 24, 0xffff87u },
  { 24, 0xffff88u },
  { 24, 0xffff89u },
  { 24, 0xffff8au },
  { 24, 0xffff8bu },
  { 24, 0xffff8cu },
  { 24, 0xffff8du },
  { 24, 0xffff8eu },
  { 24, 0xffff8fu },
  { 24, 0xffff90u },
  { 24, 0xffff91u },
  { 24, 0xffff92u },
  { 24, 0xffff93u },
  { 24, 0xffff94u },
  { 24, 0xffff95u },
  { 24, 0xffff96u },
  { 24, 0xffff97u },
  { 24, 0xffff98u },
  { 24, 0xffff99u },
  { 24, 0xffff9au },
  { 24, 0xffff9bu },
  { 24, 0xffff9cu },
  { 24, 0xffff9du },
  { 24, 0xffff9eu },
  { 24, 0xffff9fu },
  { 24, 0xffffa0u },
  { 24, 0xffffa1u },
  { 24, 0xffffa2u },
  { 24, 0xffffa3u },
  { 24, 0xffffa4u },
  { 24, 0xffffa5u },
  { 24, 0xffffa6u },
  { 24, 0xffffa7u },
  { 24, 0xffffa8u },
  { 24, 0xffffa9u },
  { 24, 0xffffaau },
  { 24, 0xffffabu },
  { 24, 0xffffacu },
  { 24, 0xffffadu },
  { 24, 0xffffaeu },
  { 24, 0xffffafu },
  { 24, 0xffffb0u },
  { 24, 0xffffb1u },
  { 24, 0xffffb2u },
  { 24, 0xffffb3u },
  { 24, 0xffffb4u },
  { 24, 0xffffb5u },
  { 24, 0xffffb6u },
  { 24, 0xffffb7u },
  { 24, 0xffffb8u },
  { 24, 0xffffb9u },
  { 24, 0xffffbau },
  { 24, 0xffffbbu },
  { 24, 0xffffbcu },
  { 24, 0xffffbdu },
  { 24, 0xffffbeu },
  { 24, 0xffffbfu },
  { 24, 0xffffc0u },
  { 24, 0xffffc1u },
  { 24, 0xffffc2u },
  { 24, 0xffffc3u },
  { 24, 0xffffc4u },
  { 24, 0xffffc5u },
  { 24, 0xffffc6u },
  { 24, 0xffffc7u },
  { 24, 0xffffc8u },
  { 24, 0xffffc9u },
  { 24, 0xffffcau },
  { 24, 0xffffcbu },
  { 24, 0xffffccu },
  { 24, 0xffffcdu },
  { 24, 0xffffceu },
  { 24, 0xffffcfu },
  { 24, 0xffffd0u },
  { 24, 0xffffd1u },
  { 24, 0xffffd2u },
  { 24, 0xffffd3u },
  { 24, 0xffffd4u },
  { 24, 0xffffd5u },
  { 24, 0xffffd6u },
  { 24, 0xffffd7u },
  { 24, 0xffffd8u },
  { 24, 0xffffd9u },
  { 24, 0xffffdau },
  { 24, 0xffffdbu },
  { 24, 0xffffdcu },
  { 24, 0xffffddu }
};

const int16_t res_huff_decode_table[][256] = {
/* 0 */
{
 32, 32, 32, 32, 32, 32, 32, 32,
 32, 32, 32, 32, 32, 32, 32, 32,
 48, 48, 48, 48, 48, 48, 48, 48,
 48, 48, 48, 48, 48, 48, 48, 48,
 49, 49, 49, 49, 49, 49, 49, 49,
 49, 49, 49, 49, 49, 49, 49, 49,
 50, 50, 50, 50, 50, 50, 50, 50,
 50, 50, 50, 50, 50, 50, 50, 50,
 51, 51, 51, 51, 51, 51, 51, 51,
 52, 52, 52, 52, 52, 52, 52, 52,
 53, 53, 53, 53, 53, 53, 53, 53,
 56, 56, 56, 56, 56, 56, 56, 56,
 57, 57, 57, 57, 57, 57, 57, 57,
 58, 58, 58, 58, 58, 58, 58, 58,
 84, 84, 84, 84, 84, 84, 84, 84,
 97, 97, 97, 97, 97, 97, 97, 97,
 101, 101, 101, 101, 101, 101, 101, 101,
 44, 44, 44, 44, 45, 45, 45, 45,
 46, 46, 46, 46, 54, 54, 54, 54,
 55, 55, 55, 55, 71, 71, 71, 71,
 77, 77, 77, 77, 83, 83, 83, 83,
 99, 99, 99, 99, 100, 100, 100, 100,
 105, 105, 105, 105, 109, 109, 109, 109,
 110, 110, 110, 110, 111, 111, 111, 111,
 112, 112, 112, 112, 114, 114, 114, 114,
 115, 115, 115, 115, 116, 116, 116, 116,
 117, 117, 117, 117, 34, 34, 47, 47,
 61, 61, 65, 65, 98, 98, 102, 102,
 103, 103, 104, 104, 108, 108, 118, 118,
 120, 120, 66, 67, 68, 69, 70, 73,
 74, 78, 79, 87, 119, 121, -19, -21,
 -22, -23, -24, -25, -26, -27, -20, -1
},
/* 1 */
{
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 43, 43, 43, 43, 43, 43, 43, 43,
 91, 91, 91, 91, 91, 91, 91, 91,
 91, 91, 91, 91, 91, 91, 91, 91,
 91, 91, 91, 91, 91, 91, 91, 91,
 91, 91, 91, 91, 91, 91, 91, 91,
 93, 93, 93, 93, 93, 93, 93, 93,
 93, 93, 93, 93, 93, 93, 93, 93,
 93, 93, 93, 93, 93, 93, 93, 93,
 93, 93, 93, 93, 93, 93, 93, 93,
 33, 33, 33, 33, 33, 33, 33, 33,
 33, 33, 33, 33, 33, 33, 33, 33,
 42, 42, 42, 42, 42, 42, 42, 42,
 42, 42, 42, 42, 42, 42, 42, 42,
 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63,
 35, 35, 35, 35, 35, 35, 35, 35,
 39, 39, 39, 39, 39, 39, 39, 39,
 62, 62, 62, 62, 62, 62, 62, 62,
 92, 92, 92, 92, 92, 92, 92, 92,
 36, 36, 36, 36, 124, 124, 124, 124,
 94, 94, 60, 64, 123, 125, 126, -2
},
/* 2 */
{
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 96, 96, 96, 96, 96, 96, 96, 96,
 163, 164, 165, 166, 167, 168, 169, 170,
 171, 172, 173, 174, 175, 176, 177, 178,
 179, 180, 181, 182, 183, 184, 185, 186,
 187, 188, 189, 190, 191, 192, 193, 194,
 195, 196, 197, 198, 199, 200, 201, 202,
 203, 204, 205, 206, 207, 208, 209, 210,
 211, 212, 213, 214, 215, 216, 217, 218,
 219, 220, 221, 222, 223, 224, 225, 226,
 227, 228, 229, 230, 231, 232, 233, 234,
 235, 236, 237, 238, 239, 240, 241, 242,
 243, 244, 245, 246, 247, 248, 249, 250,
 251, 252, 253, 254, 255, 256, -3, -4,
 -5, -6, -7, -8, -9, -10, -11, -12,
 -13, -14, -15, -16, -17, -18, -28, -29,
 -30, -31, -32, -33, -34, -35, -36, -37,
 -38, -39, -40, -41, -42, -43, -44, -45
},
/* 3 */
{
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1
},
/* 4 */
{
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3,
 3, 3, 3, 3, 3, 3, 3, 3
},
/* 5 */
{
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 4, 4, 4, 4, 4, 4, 4, 4,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5,
 5, 5, 5, 5, 5, 5, 5, 5
},
/* 6 */
{
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 6, 6, 6, 6, 6, 6, 6, 6,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7,
 7, 7, 7, 7, 7, 7, 7, 7
},
/* 7 */
{
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 8, 8, 8, 8, 8, 8, 8, 8,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9,
 9, 9, 9, 9, 9, 9, 9, 9
},
/* 8 */
{
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11,
 11, 11, 11, 11, 11, 11, 11, 11
},
/* 9 */
{
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13
},
/* 10 */
{
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 14, 14, 14, 14, 14, 14, 14, 14,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15,
 15, 15, 15, 15, 15, 15, 15, 15
},
/* 11 */
{
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 16, 16, 16, 16, 16, 16, 16, 16,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17,
 17, 17, 17, 17, 17, 17, 17, 17
},
/* 12 */
{
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 18, 18, 18,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19,
 19, 19, 19, 19, 19, 19, 19, 19
},
/* 13 */
{
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 20,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21,
 21, 21, 21, 21, 21, 21, 21, 21
},
/* 14 */
{
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 22, 22, 22, 22, 22, 22, 22, 22,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 23, 23, 23
},
/* 15 */
{
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25,
 25, 25, 25, 25, 25, 25, 25, 25
},
/* 16 */
{
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 26, 26, 26, 26,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27,
 27, 27, 27, 27, 27, 27, 27, 27
},
/* 17 */
{
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29,
 29, 29, 29, 29, 29, 29, 29, 29
},
/* 18 */
{
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 30, 30, 30, 30, 30, 30, 30, 30,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 31, 31, 31, 31, 31
},
/* 19 */
{
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 37, 37, 37, 37, 37, 37, 37, 37,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40,
 40, 40, 40, 40, 40, 40, 40, 40
},
/* 20 */
{
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 38, 38, 38, 38, 38, 38, 38, 38,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 81, 81, 81, 81, 81, 81, 81, 81,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88,
 88, 88, 88, 88, 88, 88, 88, 88
},
/* 21 */
{
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 41, 41, 41, 41, 41, 41, 41, 41,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59,
 59, 59, 59, 59, 59, 59, 59, 59
},
/* 22 */
{
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 72, 72, 72, 72, 72, 72, 72, 72,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76,
 76, 76, 76, 76, 76, 76, 76, 76
},
/* 23 */
{
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 80, 80, 80, 80, 80, 80, 80, 80,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82,
 82, 82, 82, 82, 82, 82, 82, 82
},
/* 24 */
{
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 85, 85, 85, 85, 85, 85, 85, 85,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86,
 86, 86, 86, 86, 86, 86, 86, 86
},
/* 25 */
{
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 89, 89, 89, 89, 89, 89, 89, 89,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95,
 95, 95, 95, 95, 95, 95, 95, 95
},
/* 26 */
{
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 106, 106, 106, 106, 106, 106, 106, 106,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107
},
/* 27 */
{
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 113, 113, 113, 113, 113, 113, 113, 113,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122,
 122, 122, 122, 122, 122, 122, 122, 122
},
/* 28 */
{
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128,
 128, 128, 128, 128, 128, 128, 128, 128
},
/* 29 */
{
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 129, 129, 129, 129, 129, 129, 129, 129,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130,
 130, 130, 130, 130, 130, 130, 130, 130
},
/* 30 */
{
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 131, 131, 131, 131, 131, 131, 131, 131,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132,
 132, 132, 132, 132, 132, 132, 132, 132
},
/* 31 */
{
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 133, 133, 133, 133, 133, 133, 133, 133,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134,
 134, 134, 134, 134, 134, 134, 134, 134
},
/* 32 */
{
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 135, 135, 135, 135, 135, 135, 135, 135,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136,
 136, 136, 136, 136, 136, 136, 136, 136
},
/* 33 */
{
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 137, 137, 137, 137, 137, 137, 137, 137,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138,
 138, 138, 138, 138, 138, 138, 138, 138
},
/* 34 */
{
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 139, 139, 139, 139, 139, 139, 139, 139,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140,
 140, 140, 140, 140, 140, 140, 140, 140
},
/* 35 */
{
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 141, 141, 141, 141, 141, 141, 141, 141,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142,
 142, 142, 142, 142, 142, 142, 142, 142
},
/* 36 */
{
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 143, 143, 143, 143, 143, 143, 143, 143,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144,
 144, 144, 144, 144, 144, 144, 144, 144
},
/* 37 */
{
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 145, 145, 145, 145, 145, 145, 145, 145,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146,
 146, 146, 146, 146, 146, 146, 146, 146
},
/* 38 */
{
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 147, 147,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148,
 148, 148, 148, 148, 148, 148, 148, 148
},
/* 39 */
{
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 149, 149, 149, 149, 149, 149, 149, 149,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150,
 150, 150, 150, 150, 150, 150, 150, 150
},
/* 40 */
{
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 151, 151, 151, 151, 151, 151, 151, 151,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152,
 152, 152, 152, 152, 152, 152, 152, 152
},
/* 41 */
{
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 153, 153, 153, 153, 153, 153, 153, 153,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154,
 154, 154, 154, 154, 154, 154, 154, 154
},
/* 42 */
{
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 155, 155, 155, 155, 155, 155, 155, 155,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156, 156, 156, 156, 156, 156, 156
},
/* 43 */
{
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 157, 157, 157, 157, 157, 157, 157, 157,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158,
 158, 158, 158, 158, 158, 158, 158, 158
},
/* 44 */
{
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 159, 159, 159, 159, 159, 159, 159, 159,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160,
 160, 160, 160, 160, 160, 160, 160, 160
},
/* 45 */
{
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 161, 161, 161, 161, 161, 161, 161, 161,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162,
 162, 162, 162, 162, 162, 162, 162, 162
}
};
